
import treehouse.*;
import paint.*;

/**
* @author Maxwell Planck
*
* To change this generated comment edit the template variable "typecomment":
* Window>Preferences>Java>Templates.
* To enable and disable the creation of type comments go to
* Window>Preferences>Java>Code Generation.
*/
public class PaintApplet extends THSApplet {


    /**
     * Any subclass of THSApplet needs to implement doSetup.  This functions should perform all tasks on
     * applet parameters as well as perform all needed DB calls.  Use this function as an example.
     */
    public void doSetup(int appw, int apph) {
        int mbarh = 18;
        PaintIconPanel iconBar;
        ColorPalette colhb;
        MessagePanel msgPanel;

        // refer to the python code to see how the parameters work
        int w = (new Integer(this.getParameter("initwidth"))).intValue();
        int h = (new Integer(this.getParameter("initheight"))).intValue();
        int user = (new Integer(this.getParameter("artistid"))).intValue();
        int openID = (new Integer(this.getParameter("imageid"))).intValue();

        // This object handles all opening and saving
        PaintIOPortal portal = new PaintIOPortal(this);
        portal.setUserID(user);
        int groupSettings = portal.getGroupSettings();
        
        mainCanvas = new PaintCanvas(80,30, w, h,this.getWidth(), this.getHeight(), portal, this);
        iconBar = new PaintIconPanel(8,30,56,300,this);
        colhb = new ColorPalette(8,iconBar.getCx()+iconBar.getCh()+30,53,53,this,true);
        menuBar = new THSMenuBar(appw, mbarh, this);
        msgPanel = new MessagePanel(80, apph-30, appw-100, 20);

        iconBar.setTools(groupSettings);
            
        // Note that ordering of appending does matter with when each panel is drawn
        // the latter one's appended are drawn after the first ones.
        appendPanel(mainCanvas);
        appendPanel(iconBar);
        appendPanel(menuBar);
        appendPanel(msgPanel);
        appendPanel(colhb);

        // THIS NEEDS TO BE A BETTER CHECKER
        checkBounds(iconBar.getCw() + mainCanvas.getCw() + 40, iconBar.getCh() + colhb.getCh() + msgPanel.getCh() + 40);
        iconBar.setTarget(mainCanvas);
        colhb.setTarget(mainCanvas);
        menuBar.setTarget(mainCanvas);
        mainCanvas.setTarget(msgPanel);
        
        iconBar.setFirstTool();
        initMenus();

        if (openID != -1) {
            mainCanvas.doOpen(openID);
        }
    }

    /**
     * The following function initializes the menubar options.  Refer to the PaintMenuItems class to understand
     * what the creation of each menu does.
     **/
    public void initMenus() {

        try {
            THSMenu fileMenu = menuBar.addMenu("File");
            fileMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.SAVE, mainCanvas, this));
            // fileMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.OPEN, mainCanvas, gi, this));

            THSMenu editMenu = menuBar.addMenu("Edit");
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.UNDO, mainCanvas, this));
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(THSMenuItems.SPACER, mainCanvas, this));
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.CUT, mainCanvas, this));
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.COPY, mainCanvas, this));
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.PASTE, mainCanvas, this));
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(THSMenuItems.SPACER, mainCanvas, this));
            editMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.SHOWCLIPBOARD, mainCanvas, this));

            //THSMenu testMenu = menuBar.addMenu("Test");
            //testMenu.addMenuItem(PaintMenuItems.makeMenuItem(PaintMenuItems.TEST, mainCanvas, this));
        } catch (RuntimeException re) {
            System.err.println("One of the menu items added wasn't properly instantiated either due to a typo or the fact that it hasn't been added to the PaintMenuItems class yet.");
            re.printStackTrace();
        }

    }

    public static void main(String[] args) {
        // note, runTHSApplet does not yet work
        runTHSApplet(new PaintApplet(), "Paint");
    }
}
