//
//  PaintBrush.java
//  Treehousestudio
//
//  Created by Maxwell Planck on Tue Nov 26 2002.
//  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
//

package paint;
import java.awt.*;
import java.applet.*;
import java.awt.image.*;

/**
 * This abstract class is the template for a PaintBrush.  Simply, a PaintBrush is a tool that leaves a mark
 * of particular size and type on the canvas whenever it's paint method is called at a specified location.
 */
public abstract class PaintBrush {
    int sz;

    public PaintBrush(int _sz) {
        setsize(_sz);
    }
    
    public PaintBrush setsize(int s) {
        sz = s;
        return this;
    }

    /**
     * These are factory methods for making unique brushes
     */
    public static PaintBrush makeSquareBrush(int size) {
        return new squarebrush(size);
    }

    public static PaintBrush makeRoundBrush(int size) {
        return new roundbrush(size);
    }

    public static PaintBrush makeImageBrush(int size, Image im, Applet app) {
        return new imbrush(size, im, app);
    }

    public abstract void paint(Graphics g, int x, int y);
}


/**
 * This is the simplest kind of brush- a squarebrush that draws of square of particular size when it's paint
 * method is called
 */
class squarebrush extends PaintBrush {

    public squarebrush(int _n) {
        super(_n);
    }
    
    public void paint(Graphics g, int x, int y) {
        g.fillRect(x,y,sz,sz);
    }
    
}

/**
 * This PaintBrush places an image at the location the that paint is called.  The image drawn is specified once the
 * brush is instantiated.
 */
class imbrush extends PaintBrush {
    int n;
    Applet app;
    Image im;

    public imbrush(int _n,Image _im,Applet _app)
    {
        super(_n);
        im = _im;app =_app;
    }

    public void paint(Graphics g, int x, int y)
    {
        g.drawImage(im,x,y,app);
    }
}

/**
 * This is a roundbrush- a round filled circle is drawn whenever the paint method is called
 */
class roundbrush extends PaintBrush {
    int n;

    public roundbrush(int _n)
    {
        super(_n);
    }
    public void paint(Graphics g, int x, int y)
    {
        g.fillOval(x,y,sz,sz);
    }
}
