//
//  PaintCanvas.java
//  Treehousestudio
//
//  Created by john maeda on Thu Aug 15 2002.
//  Copyright (c) 2001 __MyCompanyName__. All rights reserved.
//


package paint;
import java.awt.*;
import java.applet.*;
import java.util.*;
import java.sql.*;

import treehouse.*;

/**
* This class is the painting canvas for the painting applet.
 */
public class PaintCanvas extends THSCanvas
{

    int mode;
    Image imstk;
    Graphics gstk;

    boolean pushedp = false;

    /**
        * This method is called from anywhere in the applet
     **/
    public void message(String fromWho, Object odat)
    {
        if (fromWho.equals("FGColor Change")) {
            fgcol = (Color) odat;
            return;
        }
        if (fromWho.equals("BGColor Change")) {
            bgcol = (Color) odat;
            return;
        }
        if (fromWho.equals("Doubleclick")) { // double click
            selectedTool = (THSGeneralTool) odat;
            System.out.println("doubleclick");
            if (selectedTool.getString() == "Eraser") {
                clearbuffer();
            }
            return;
        }
        // if you can't handle the message, try the superclass
        super.message(fromWho, odat);
    }


    Image im;
    Graphics gi;

    Image imundo;
    Graphics giu;

    Image imtmp;
    Graphics git;

    Applet app;

    int dimlw = 4;
    int limitWidth = 0;
    int limitHeight = 0;
    Color fgcol=Color.black;;
    Color bgcol=Color.white;

    THSWriteCanvas imdat =null;
    PaintIOPortal portal;

    /**
        * This is the main component of the TreeHouse Draw application.  This is essentially the canvas in which all
     * drawing, painting, erasing, etc. occurs.
     */
    public PaintCanvas(int _x, int _y, int _w, int _h, int lw, int lh, PaintIOPortal portal, Applet _app)
    {
        super("Paint Canvas", _x, _y, _w, _h);
        limitHeight = lh;
        limitWidth = lw;

        app = _app;
        im = app.createImage(cw,ch);
        gi = im.getGraphics();

        imtmp = app.createImage(cw,ch);
        git = imtmp.getGraphics();

        imundo = app.createImage(cw,ch);
        giu = imundo.getGraphics();

        newbuffer();

        // build brushes
        MediaTracker mt = new MediaTracker(app);
        Image im;
        try {
            mt.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }

        this.portal = portal;
    }
        
        
    /** --------------------------------
        * OBSERVERS
        *  -------------------------------- **/

    public Graphics getGraphics() {
        return gi;
    }

    public Image getImage() {
        return im;
    }

    public Color getFGColor() {
        return fgcol;
    }

    public Color getBGColor() {
        return bgcol;
    }

    public Applet getApp() {
        return app;
    }

    /** --------------------------------
        * Modifiers
        *  -------------------------------- **/

    public void clearbuffer()
    {
        cacheUndo();
        gi.setColor(Color.white);
        gi.fillRect(0,0,cw,ch);
        gi.setColor(Color.black);
    }

    public void newbuffer()
    {
        gi.setColor(Color.white);
        gi.fillRect(0,0,cw,ch);
        gi.setColor(Color.black);

        giu.setColor(Color.white);
        giu.fillRect(0,0,cw,ch);
        giu.setColor(Color.black);
    }

    public void push()
    {
        if (pushedp) {
            err("can't push more than once");
            return;
        }

        // swap buffers
        imstk = im; gstk = gi;

        git.drawImage(im,0,0,app);
        im = imtmp; gi = git;
        pushedp = true;
    }

    public void pop()
    {
        if (!pushedp) return;
        // restore buffers
        im = imstk; gi = gstk;
        gi.drawImage(imtmp,0,0,app);
        pushedp = false;
    }

    public void recover() {
        gi.drawImage(imstk,0,0,app);
    }

    public void cacheUndo() {
        giu.drawImage(im,0,0,app);
    }

    public void doUndo()
    {
        git.drawImage(im,0,0,app);
        gi.drawImage(imundo,0,0,app);
        giu.drawImage(imtmp,0,0,app); // swap images effectively
    }

    /**
        * The following two methods handle the saving and opening of files within the DB
     */

    public void doSave()
    {
        // This rasterizes the drawn image into a byte array
        // that is saved into the db as binary information
        imdat = new PaintImageRasterizer(cw,ch);
        byte[] b = imdat.rasterizeImage(im);
        portal.doSave(b);
        // imdat.saveThumbnail(im);
        String dat = MessagePanel.SAVE + "Canvas saved successfully";
        target.message("Main Canvas", dat);
    }

    public void doOpen(int id)
    {

        try {
            byte[] image = portal.doOpen(id);
            Image im;

            // Extract the byte information from the information provided and draw it
            
            imdat = new PaintImageRasterizer();
            im = imdat.extractImage(image,app);
            this.doResize(imdat.getWidth(), imdat.getHeight());
            gi.drawImage(im,0,0,app);
            cacheUndo(); //so that undo won't blank out the canvas
            
        } catch (Exception e) {
            target.message("Main Canvas", MessagePanel.ERROR + e.getMessage());
        }
    }

    public void doTest() {
        target.message("Main Canvas", MessagePanel.ERROR + "-" + portal.getUserID() + " " + portal.getCanvasID());
    }


    /**
     * This method resizes the canvas to any particular positive value.  Note that it does not account for the bounds of
     * the applet, that needs to be handled by whatever method calls this one.
     */
    public void doResize(int w, int h) {
        if ((w < 1) && (h < 1)) {
            return; //nothing happens
        }
        cw = w;
        ch = h;
        im = app.createImage(cw,ch);
        gi = im.getGraphics();
        imtmp = app.createImage(cw,ch);
        git = imtmp.getGraphics();
        imundo = app.createImage(cw,ch);
        giu = imundo.getGraphics();
        System.out.println("Should allow the painting region to be resized");
    }

    public void cleanUpIfMouseDown(Event ev, int mx, int my) {
        if (selectedTool.getString().equals("Polygon")) {
            selectedTool.mouseDown(mx, my, this);
            selectedTool.mouseDown(mx, my, this);
            // doubleclicks
        }
    }
    
    public void paint(Graphics g)
    {
        int i;

        Rectangle r = g.getClipBounds();
        if(r == null) r = new Rectangle(0, 0, 2000, 2000);
        g.setClip(cx, cy, cw,ch);
        g.clipRect(cx, cy, cw, ch);
        g.drawImage(im,cx,cy,cw,ch,app);
        g.setClip(r.x,r.y,r.width,r.height);

        // outline the canvas
        g.setColor(Color.black);
        g.drawRect(cx,cy,cw+1,ch+1);
    }

}
