//
//  THSWriteCanvas.java
//  Treehousestudio
//
//  Created by Maxwell Planck on Thu Jan 23 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

package paint;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.applet.*;

import treehouse.*;

/**
* This wrapper class rasterizes the image drawn on the screen into a byte array.  Currently, this method
 * saves the image by using an index table.  This implementation saves space but costs a little processing time
 * on saving and opening.
 */
public class PaintImageRasterizer extends THSWriteCanvas {

    byte []vs = null;
    int []idat = null;
    boolean indexing;
    int h_offset;
    int h_offset_i;
    
    public PaintImageRasterizer(int _w, int _h) {
        super(_w, _h);
        setToIndexRasterizing(false);
        // Header Offset of bytes gives room to attach width and height
        h_offset = 5;
        h_offset_i = 4+256*3;
    }

    public PaintImageRasterizer() {
        super();
        setToIndexRasterizing(false);
        // Header Offset of bytes gives room to attach width and height
        h_offset = 5;
        h_offset_i = 4+256*3;
    }

    public void setToIndexRasterizing(boolean b) {
        indexing = b;
    }

    public byte[] rasterizeImage(Image img) {
        if (!convertToBytes(img)) {
            System.out.println("Rasterization didn't work");
            return null;
        }

        if (vs == null) {
            vs = new byte[h_offset + w*h*3];
        }

        byte[] vsi = new byte[h_offset + w*h];

        vs[0] = (byte) 0xF0; // The header tag for a paint format
        
        vs[1] = (byte) ((w >> 8) & 0xFF);
        vs[2] = (byte) (w & 0xFF);
        vs[3] = (byte) ((h >> 8) & 0xFF);
        vs[4] = (byte) (h & 0xFF);

        System.out.println("Width of Image: " +(((vs[1] & 0xFF) << 8) | (vs[2] & 0xFF)) + " px");
        System.out.println("Height of Image:  " +(((vs[3] & 0xFF) << 8) | (vs[4] & 0xFF)) + " px");

        int[] pxObserver = new int[256];
        // Holds the header of the array index
        int pxpointer = 0;
        // Reports the position of the pixel in the index array
        int pxpos = 0;

        for  (int  j  =  0;  j  <  h;  j++)  {
            for  (int  i  =  0;  i  <  w;  i++)  {
                // look at the pixel
                //System.out.print(pixels[j*w+i] +" ");
                //                  handleSinglePixel(x+i, y+j, pixels[j * w + i]);
                int pixel =  graster[j * w + i];
                int  alpha  =  (pixel  >>  24)  &  0xff;
                int  red      =  (pixel  >>  16)  &  0xff;
                int  green  =  (pixel  >>    8)  &  0xff;
                int  blue    =  (pixel            )  &  0xff;

                // the following code maintains how many different colors are in the image
                // this creates the image index

                if (indexing) {
                    // assume that this is a new color
                    boolean isNewColor = true;
                    for (int k = 0; k <= pxpointer; k++) {
                        if (pxObserver[k] == pixel) {
                            pxpos = k;
                            isNewColor = false;
                            break;
                        }
                    }
                    if (isNewColor) {
                        pxpointer++;
                        pxpos = pxpointer;
                        pxObserver[pxpointer] = pixel;
                        System.out.println(pixel + " ---- " +pxpointer);

                        vsi[pxpointer*3+0] = (byte)(0xff&red);
                        vsi[pxpointer*3+1] = (byte)(0xff&green);
                        vsi[pxpointer*3+2] = (byte)(0xff&blue);
                    }
                }

                vsi[h_offset + i+j*w] = (byte)(0xff&pxpos);

                vs[h_offset+(i*3+j*w*3+0)] = (byte)(0xff&red);
                vs[h_offset+(i*3+j*w*3+1)] = (byte)(0xff&green);
                vs[h_offset+(i*3+j*w*3+2)] = (byte)(0xff&blue);
            }
        }

        return vs;
    }

    /**
     * This method should not be accessed by any method that is attempting to save or load, special tags must
     * be placed on the bytes to distinguish it before being placed in the db.  Only use for internal
     * manipulation.
     */
    public boolean convertToBytes(Image img)
    {

        if ((w<1) && (h<1)) {
            System.err.println("Width or height dimensions are not correct");
            return false;
        }
        
        if (graster==null) graster = new int[w*h];
        if (graster.length<w*h) {graster=null;graster = new int[w*h];}

        int[]  pixels  =  graster;
        PixelGrabber  pg  =
            new  PixelGrabber(img,  0,  0,  w,  h,  pixels,  0,  w);
        try  {
            pg.grabPixels();
        }  catch  (InterruptedException  e)  {
            System.err.println("interrupted  waiting  for  pixels!");
            return false;
        }
        if  ((pg.status()  &  ImageObserver.ABORT)  !=  0)  {
            System.err.println("image  fetch  aborted  or  errored");
            return false;
        }
        // System.out.println("rasterized");
        return true;
    }

    public Image extractImage(byte []b, Applet app)
    {
        DirectColorModel dm;
        MemoryImageSource ms;
        int i,j,k;

        if ((b.length == 0) || (b[0] != (byte) 0xF0)) {
            throw new IllegalFormatException("The data specified is not of type paint or not available");
        }

        this.setWidth((((b[1] & 0xFF) << 8) | (b[2] & 0xFF)));
        this.setHeight((((b[3] & 0xFF) << 8) | (b[4] & 0xFF)));
        
        System.out.println("Width of Image: " + w + " px");
        System.out.println("Height of Image:  " + h + " px");

        // should just do a newimage and cache the image ...
        // PROBLEM ***************** makes too many images

        dm = new DirectColorModel(24,0xff0000,0xff00,0xff);
        if (idat == null) idat = new int[w*h];

        System.out.println(b.length);
        for(j=0,k=0;j<h;j++)
            for(i=0;i<w;i++,k++) {
                //System.out.println("\b"+k+" "+i+" "+j+" || "+ (i*3+j*3*w+h_offset));
                try {
                    idat[k] = (((0xff&b[i*3+j*3*w+h_offset])<<16) | ((0xff&b[i*3+j*3*w+1+h_offset])<<8) | (0xff&b[i*3+j*3*w+2+h_offset]));
                } catch (ArrayIndexOutOfBoundsException aioobe) {
                    System.out.println("\b"+k+" "+i+" "+j+" || "+ (i*3+j*3*w+h_offset));
                }
            }

        ms = new MemoryImageSource(w,h,dm,idat,0,w);
        ms.setAnimated(false);
        datim = app.createImage(ms);
        return datim;
    }
}