package treehouse;

import java.applet.Applet;

/**
* @author tom
*
*/

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

import paint.*;

/**
* HOW TO ADD A MENU ITEM (cont.)
*
* If you haven't followed the instructions in the THSMenuItems class, go there before reading on.
*
* All setup that you need to perform should be implemented in the doSetup() method that is declared
* abstract in this class.  Each new applet of the Treehouse Studio should define their own doSetup
* and it is within this method that menu creation occurs.  It is also within this method that you
* you instantiate and setup all panels that will lie within your applet.  See PaintApplet for an example.
*
* Now that you've made your own menu items with functionality and everything, it's time to place them in
* a menu to be displayed. To make a new menu category, like "File" and "Edit", instantiate a new THSMenu
* by telling the menuBar object to add a Menu with the name you would like.  The addMenu() method then
* returns a THSMenu object which you can populate with menu items.  To add a menu item, just use the addMenuItem()
* method of the THSMenu object, passing a newly instantiated menu item as a parameter.  To create a menu item, use the
* static makeMenuItem() method of your THSMenuItems extended class, calling the static variable associated with
* your added menu item.  Keep in mind that it DOES matter the order you add the menu items.  Use the spacer
* menu item to divide up the specific sections within the menu categories.  Make sure to follow the examples
* provided in the PaintApplet class, passing the right parameters into the makeMenuItem() method so that a
* proper menu item (with the potential to actually do something) is produced by the static factory method.
*
* Once you've done all of this, the menu item should appear in the menu bar the next time you compile and run
* the applet.
*/

public abstract class THSApplet extends Applet {

    /***********************************************************************************
    ************************************************************************************
    ************************************************************************************
    **
    ** All of the following code can be modified to update the current Tree House Studio
    ** Applet so functionality can be added to the menu bar
    **
    ************************************************************************************
    ************************************************************************************
    ************************************************************************************/


    public abstract void doSetup(int cbw, int cbh);

    /************************************************************************************
    ************************************************************************************
    ************************************************************************************
    **
    ** Everything else beyond this point is internal and shouldn't be changed unless
    ** you want to change an underlying aspect of the Applet.  Do use the rest of this code
    ** to understand how to add to the Applet.
    **
    ************************************************************************************
    ************************************************************************************
    ************************************************************************************/
    boolean isSetup = false;

    // Two sets of graphics
    //protected Graphics gi;
    //Graphics cg;

    int cbw, cbh;

    Vector panels = new Vector();
    Hashtable bh = new Hashtable();

    THSPanel mhitpanel;
    private int mbarh = 18;

    // These are the main components of the applet
    protected THSCanvas mainCanvas;
    protected THSMenuBar menuBar;
    protected THSIconPanel iconBar;

    Font menufont;
    FontMetrics fm;

    // These are the static varmenuBariables that can be referenced by a call to reportMousePos
    protected static int mx = 0, my = 0;

    public void init() {
        setLayout(null);
    }

    public void appendPanel(THSPanel a)
    {
        //this.add(a);
        panels.addElement(a);
        bh.put(a.cs,a);
    }

    /**
        * This method returns the THSPanel that the mouse is within considering that it searches down to the most
        * specific child and if overlapping occurs, returns the first one to be added to the list
        * of panels
        **/
    public THSPanel hitpanel(int x, int y)
    {
        int i;
        THSPanel a, res;
        for(i=0;i<panels.size();i++) {
            a = (THSPanel) panels.elementAt(i);
            if (a.insidep(x,y)) {
                return a;
            }
        }
        return null;
    }

    /** --------------------------------
        * These methods are all used to handle mouse actions
        *  -------------------------------- */

    public boolean mouseDown(Event ev, int x, int y) {
        mx = x;
        my = y;

        /*if (mhitpanel != null) {
            // This is just to make sure everything is clean
            mhitpanel.cleanUpIfMouseDown(ev, x, y);
        }*/
        
        mhitpanel = hitpanel(x,y);

        if (mhitpanel != null) {
            mhitpanel.mouseDown(ev,x,y);
            this.repaint();
            //redraw();
            //displayGraphics();
        }

        return true;
    }

    public boolean mouseDrag(Event ev, int x, int y) {
        mx = x;
        my = y;

        if (mhitpanel != null) {
            mhitpanel.mouseDrag(ev,x,y);
            this.repaint();
        }
        return true;
    }

    public boolean mouseUp(Event ev, int x, int y) {
        mx = x;
        my = y;
        if (mhitpanel!=null) {
            /*if (mhitpanel.getString() == "Menu") {
            redraw();
            }
            menuBar.mouseUp(ev, x, y);
            displayGraphics();
            return true;
            if (mhitpanel.getString() == "Paint Canvas") {
                ((PaintCanvas) mhitpanel).mouseUp(ev,x,y);
                return true;
            }
            if (mhitpanel.getString() == "Menu Bar") {
                return true;
            }*/

            mhitpanel.mouseUp(ev,x,y);
            this.repaint();
            //redraw();
            //displayGraphics();
        }

        mhitpanel = null;
        return true;
    }

    public boolean mouseMove(Event ev, int x, int y) {
        mhitpanel = hitpanel(x,y);
        if (mhitpanel != null) {
            if (mhitpanel.getString() == "Paint Canvas") {
                ((THSCanvas) mhitpanel).mouseMove(ev,x,y);
                this.repaint();
                return true;
            }
        }
        return false;
    }

    public static int[] reportMousePos() {
        int[] res = {mx, my};
        return res;
    }

    /**----------------------------------
        * This method handles all key actions
        * ---------------------------------- */

    public boolean keyDown(Event ev, int k) {
        if(mainCanvas.keyDown(ev,k))
        {
            this.repaint();
            //redraw();
            //displayGraphics();
        }
        return true;
    }


    /** --------------------------------
        * These methods are all used to handle the drawing and buffering of the drawn image
        *  -------------------------------- */

    /*public void redraw() {
        gi.setColor(Color.gray);
        gi.fillRect(0,0,cbw,cbh);
        gi.setColor(Color.black);
        gi.drawRect(0,0,cbw,cbh);

        for (int i = 0; i < panels.size(); i++) {
            THSPanel currPanel = (THSPanel) panels.elementAt(i);
            currPanel.repaint();
        }
    }

    public void displayGraphics() {
        cg.drawImage(im,0,0,this);
    }*/

    public void paint(Graphics g) {

        Rectangle cb = this.bounds();
        cbw = cb.width;
        cbh = cb.height;
        
        if (!isSetup) {
            // This should only happen once
            doSetup(cbw, cbh);
            isSetup = true;
        }
        /*if (cbw>1) { // There needs to be at least a pixel's width of space
            if (im==null) {
                // This instantiates all of the items within the applet if the image has not been created
                // yet
                im = createImage(cbw,cbh);
                gi = im.getGraphics();
                //cg = g.create();

                doSetup(cbw, cbh);
            }
        } else return;*/

        g.setColor(Color.gray);
        g.fillRect(0,0,cbw,cbh);
        g.setColor(Color.black);
        g.drawRect(0,0,cbw,cbh);
        //gi.drawImage(im,0,0,this);

        //redraw();
        //displayGraphics();
        for (int i = 0; i < panels.size(); i++) {
            THSPanel currPanel = (THSPanel) panels.elementAt(i);
            currPanel.paint(g);
        }
    }

    public void checkBounds(int width, int height) {
        if (width > cbw) {
            cbw = width;
        }
        if (height > cbh) {
            cbh = height;
        }
    }

    /**
        * Method runTHSApplet.
        * @param applet - the applet to run in a standalone window
        * @param title - string for the window title
        * 
        * This method doesn't work because THS always tries to pull icon
        * stuff using getCodeBase(). If this did work, I would propose
        * moving it into the base class so everyone could make a simple main.
        * 
        */
    public static void runTHSApplet(THSApplet applet, String title) {

        // this does not work because of getCodeBase icon stuff in THS
        Frame frame = null;
        frame = new Frame();
        frame.addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent e)
        {
                Frame f = (Frame)e.getSource();
                f.setVisible(false);
                f.dispose();
                System.exit(0);
        }

        });

        frame.setTitle(title);
        frame.add(applet, BorderLayout.CENTER);
        applet.init();
        applet.start();
        LayoutManager lay = applet.getLayout();
        Dimension d;
        frame.setBackground(Color.darkGray);
        frame.setVisible(true);
        if(lay != null)
            d = lay.preferredLayoutSize(applet);
        else 
            d = new Dimension(400, 300);
        Insets is = frame.getInsets();
        d.setSize(d.width + is.left + is.right, d.height+is.top + is.bottom);
        frame.setSize(d);
        d = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation((d.width - frame.getSize().width) / 2, (d.height - frame.getSize().height) / 2);
        frame.show();
    }
}

