//
//  THSIOPortal.java
//  Treehousestudio
//
//  Created by Maxwell Planck on Thu Jan 23 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

package treehouse;
import java.sql.*;
import java.applet.*;

/**
 * Refer to this class to understand how saving and loading works with the SQL database.
 *
 * More explicitly, doOpen() and doSave() are the two functions that load and save respectively.
 *
 * doOpen() takes in an integer primary key for a particular canvas and then returns the byte information
 * saved for that entry.
 *
 * doSave() takes in a byte array that represents all information on the canvas and then writes it to the DB
 * along with the current user id associated with that canvas
 *
 * Any subclasses of THSIOPortal should add to the functionality of these two functions and then call
 * the function explicitly at the end of the function, making it a true subtype.
 */

public class THSIOPortal {

    private Applet app;
    
    // Default values for DB
    protected String dbuser = "webuser";
    protected String dbpassword = "";
    protected String db = "thstudio";
    protected String host = "localhost";
    
    protected int currUserID;
    protected int currGroupID;
    protected String currUser;

    protected Statement stmt;
    protected Connection c;
    protected ResultSet RS;
    protected String table;
    protected int openCanvas;
    
    public THSIOPortal(Applet app) {
        this.app = app;
        openCanvas = -1;
        currUserID = -1;
        currGroupID = -1;
    }

    public void setDBUser(String u) {
        dbuser = u;
    }

    public void setDBPassword(String p) {
        dbpassword = p;
    }

    public void setHost(String h) {
        host = h;
    }
    
    public void setDB(String d) {
        db = d;
    }

    public void setTable(String table) {
        this.table = table;
    }
    
    public void setUserName(String username) {
        if (authUserName(username)) {
            currUser = username;
        }
    }

    public void setUserID(int userid) {
        if (authUserID(userid)) {
            currUserID = userid;
        }
    }

    /**
     * Returning a -1 means the user has not been set yet for this portal.  The proper integer is returned
     * otherwise.
     */
    public int getUserID() {
        return currUserID;
    }

    /**
     * Returning a -1 means the canvas has not been saved yet.  The proper integer is returned
     * otherwise.
     */
    public int getCanvasID() {
        return openCanvas;
    }

    /**
     * Returning a -1 if the user has not been set yet, the group settings in integer form otherwise
     */
    public int getGroupSettings() {
        int settings = -1;
        if (currGroupID != -1) {
            try {
                stmt = getStatement();
                ResultSet RS = stmt.executeQuery("Select * from groups where id='"+currGroupID+"'");
                RS.next();
                settings = RS.getInt("SETTINGS");
                
                RS.close();
                stmt.close();
                c.close();
            } catch (SQLException e) {
                System.out.println("SQLException: " + e.getMessage());
                System.out.println("SQLState:     " + e.getSQLState());
                System.out.println("VendorError:  " + e.getErrorCode());
            }
        }
        return settings;
    }

    private boolean authUserName(String user) {
        try {
            stmt = getStatement();
            ResultSet RS = stmt.executeQuery("Select * from users where username='"+user+"'");
            RS.next();
            int tempuID = RS.getInt("ID");
            int tempgID = RS.getInt("GROUPID");
            
            RS.close();
            stmt.close();
            c.close();
            currUserID = tempuID;
            currGroupID = tempgID;
            return true;
        } catch (SQLException e) {
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
            return false;
        }
    }

    private boolean authUserID(int userid) {
        try {
            stmt = getStatement();
            ResultSet RS = stmt.executeQuery("Select * from users where id='"+userid+"'");
            RS.next();
            String tempUname = RS.getString("USERNAME");
            int tempgID = RS.getInt("GROUPID");

            RS.close();
            stmt.close();
            c.close();
            currUser = tempUname;
            currGroupID = tempgID;
            return true;
        } catch (SQLException e) {
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
            return false;
        }

    }

    /**
     * Saves the current canvas (passed as a byte array) into the database memory given the username and id.
     * Note that this same save method can be used by any type of applet since the bytes it is passing through
     * are independent of their format.
     */
    public Statement getStatement() {

        try {
            Class.forName("org.gjt.mm.mysql.Driver").newInstance();
        } catch (Exception e) {
            System.out.println("Unable to load Driver");
            e.printStackTrace();
        }

        try {
            if (dbpassword != "")
                c = DriverManager.getConnection("jdbc:mysql://" + host + "/" + db + "?user="+dbuser+"&password="+dbpassword);
            else
                c = DriverManager.getConnection("jdbc:mysql://" + host + "/" + db + "?user="+dbuser);
            
            stmt = c.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);
            stmt.setEscapeProcessing(true);
        } catch (SQLException e) {
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
        }

        return stmt;

    }
    
    public byte[] doOpen(int id) {
        byte[] data = new byte[0];
        try {
            Statement stmt = getStatement();
            ResultSet RS = stmt.executeQuery("Select * from " + table + " where id="+id);
            RS.next();
            data = RS.getBytes("DATA");
            int checkID = RS.getInt("ARTISTID");
            if (checkID != currUserID) {
                throw new ForbiddenException("This user is not allowed to access this saved canvas");
            }
            openCanvas = RS.getInt("ID");
            RS.close();
            stmt.close();
            c.close();
        } catch (SQLException e) {
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
        }
        return data;
    }

    public boolean doSave(byte[] b) {
        try {
            Statement stmt = getStatement();

            if (openCanvas == -1) {
                ResultSet RS = stmt.executeQuery("Select * from " + table);
                RS.moveToInsertRow();
                RS.updateInt("ARTISTID", currUserID);
                RS.updateInt("GROUPID", currGroupID);
                RS.updateBytes("DATA", b);
                RS.insertRow();
                RS.last();
                openCanvas = RS.getInt("ID");
                System.out.println(openCanvas);
                System.out.println(">>>>>>> Inserted " + b.length + " bytes into the DB at " + new java.util.Date());
                RS.close();
            }
            else {
                ResultSet RS = stmt.executeQuery("Select * from " + table + " where id='" + openCanvas + "'");
                RS.next();
                RS.updateBytes("DATA", b);
                System.out.println(">>>>>>> Updated " + b.length + " bytes into the DB at " + new java.util.Date());
                RS.close();
            }
            stmt.close();
            c.close();
            
        } catch (SQLException e) {
            System.out.println("SQLException: " + e.getMessage());
            System.out.println("SQLState:     " + e.getSQLState());
            System.out.println("VendorError:  " + e.getErrorCode());
        }
        return false;
    }
}
