//
//  THSIconPanel.java
//  treehousedraw_max
//
//  Created by Maxwell Planck on Fri Oct 25 2002.
//  Copyright (c) 2002 __MyCompanyName__. All rights reserved.
//

package treehouse;
import java.awt.*;
import java.applet.*;
import java.net.URL;
import java.util.*;
import java.awt.image.*;

/**
 * This THSPanel holds the tool menu for selecting what kind of painting tool one wants
 * to use
 */

public abstract class THSIconPanel extends THSPanel // icon area
{

    /***********************************************************************************
    ************************************************************************************
    ************************************************************************************
    **
    ** The initTools function can be modified in the subclass of THSIconPanel to determine
    ** what tools are used in the icon panel.
    **
    ************************************************************************************
    ************************************************************************************
    ************************************************************************************/

    public abstract void initTools();

    /************************************************************************************
    ************************************************************************************
    ************************************************************************************
    **
    ** Everything else beyond this point is internal and shouldn't be changed unless
    ** you want to change an underlying aspect of the Applet.  Do use the rest of this code
    ** to understand how to add to the Applet.
    **
    ************************************************************************************
    ************************************************************************************
    ************************************************************************************/
    
    Applet app;
    
    //   final static String icons[] =
    int rows, cols;
    final int iconW = 27, iconH = 23;
    
    int selx = 1, sely = 1;
    int touchedToolind;
    public Vector tools;
    public THSGeneralTool selectedTool;
    
    long lastt=0;
    int lastsel;

    MediaTracker mt;
    Vector icons;
    Vector iconStrings;

    boolean[] enabledTools;
    int toolcount = 0;
    
    public THSIconPanel(int _x, int _y, int _w, int _h, Applet _app)
    {
        super("Icon Panel",_x,_y,_w,_h);
        app = _app;
        mt = new MediaTracker(app);

        tools = new Vector();
        iconStrings = new Vector();
        initTools();
        enabledTools = new boolean[toolcount];
        
        for (int i = 0; i < enabledTools.length; i++) {
            enabledTools[i] = true;
        }

        icons = retrieveIcons();
        cols = 2; // restricts the row count
        rows = (int) Math.ceil(icons.size()/2.0); //as many rows as are needed

        // This updates the super variables
        ch = rows*iconH;
        cw = cols*iconW;
    }

    public THSGeneralTool getSelectedTool() {
        return selectedTool;
    }
    

    public void disableTool(int ind) {
        enabledTools[ind] = false;
        Image im = (Image) icons.elementAt(ind);
        GrayFilter filter = new GrayFilter();
        im = app.createImage(new FilteredImageSource(im.getSource(), filter));
        icons.setElementAt(im, ind);
    }

    public void addTool(THSGeneralTool tool) {
        tools.addElement(tool);
        toolcount++;
    }

    public void setFirstTool() {
        int pickedTool;
        for (pickedTool=0; pickedTool<enabledTools.length; pickedTool++) {
            if (enabledTools[pickedTool]) {
                break;
            }
        }
        selx = pickedTool%cols;
        sely = pickedTool/cols;
        if (target != null) {
            target.message("Icon Panel", tools.elementAt(pickedTool));
        } else {
            throw new RuntimeException("An initial tool must be initialized within the Treehousestudio class");
        }
    }


    // Don't mind the methods behind this line
    // -----------------------------------------
	public Image loadImage(String name) {
		Image im = null;
		
		try {
			im = app.getImage(app.getCodeBase(), name);
		}	
		catch (Exception e) {
			try {
                                URL url = this.getClass().getResource(name);
				ImageProducer ip = (ImageProducer)url.getContent();
				im = Toolkit.getDefaultToolkit().createImage(ip);
			}
			catch (Exception e2) {
				System.out.println("Sorry, could not load: " + name);
				System.out.println(e2);
				e2.printStackTrace();
			}
		
//			Toolkit t = Toolkit.getDefaultToolkit();
//			im = t.getImage(name);
//			System.out.println("Image: " + im.getHeight(null));
//			System.out.println("Error loading image: " + e);
		}
		return im;
	}


      private Vector retrieveIcons() {
        Vector res = new Vector();
        for (int i = 0; i < tools.size(); i++) {
            String icon = ((THSGeneralTool) tools.elementAt(i)).getIconImage();
            Image im = loadImage("icons/" + icon);
            im = im.getScaledInstance(iconW,iconH,Image.SCALE_SMOOTH);
            if (im.getHeight(app) != -1) {
                throw new RuntimeException("The icon referring to image " + icon + " was not loaded properly");
            }
            mt.addImage(im,i);
            res.addElement(im);
        }

        try {
            mt.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return res;
    }

/*
 *     private Vector retrieveIcons() {
        Vector res = new Vector();
        for (int i = 0; i < tools.size(); i++) {
            String icon = ((THSGeneralTool) tools.elementAt(i)).getIconImage();
            Image im = app.getImage(app.getCodeBase(), "icons/" + icon);
            im = im.getScaledInstance(iconW,iconH,Image.SCALE_SMOOTH);
            if (im.getHeight(app) != -1) {
                throw new RuntimeException("The icon referring to image " + icon + " was not loaded properly");
            }
            mt.addImage(im,i);
            res.addElement(im);
        }

        try {
            mt.waitForAll();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return res;
    }
*/
    
    public void paint(Graphics g) {
        for (int i = 0; i < icons.size(); i++) {
            if (icons.elementAt(i) != null) {
                Image currImage = (Image) icons.elementAt(i);
                g.drawImage(currImage,cx+(i%2*(iconW)),cy+(int) (Math.floor(i/2))*(iconH), app);
            }
        }
        g.setColor(Color.blue);
        g.drawRect(cx+1+selx*iconW,cy+1+sely*iconH,iconW-3,iconH-3);
    }
    
    public boolean mouseDown(Event ev, int mx, int my) {
        int tempsely, tempselx, tempTouchedToolind;
        mx-=cx; my-=cy;
        // assign temporary values until it is determined that the selection is valid
        tempsely = Math.min(my/iconH, rows-1);
        tempselx = Math.min(mx/iconW, cols-1);
        tempTouchedToolind = tempselx+tempsely*cols;
        try {
            THSGeneralTool touchedTool = (THSGeneralTool) tools.elementAt(tempTouchedToolind);
            if ((!enabledTools[tempTouchedToolind]) || (touchedTool == null)) {
                return true; // the tool is disabled or not there
            }
            // This tool can be selected- assign over all temporary values
            selectedTool = touchedTool;
            touchedToolind = tempTouchedToolind;
            selx = tempselx;
            sely = tempsely;

            long t = System.currentTimeMillis();

            if (t-lastt<500 && lastsel == touchedToolind) {
                target.message("Doubleclick", selectedTool);
            } else
                target.message("Icon Panel", selectedTool);
            lastt = t;
            lastsel = touchedToolind;
            return true;
        } catch (ArrayIndexOutOfBoundsException e) {
            return false; // there is not tool at this blank spot
        }
    }

    public boolean mouseDrag(Event ev, int mx, int my) {
        return false;
    }
    public boolean mouseUp(Event ev, int mx, int my) {
        return false;
    }

    public boolean keyDown(Event ev, int k) {
        return false;
    }

    public void message(String fromWho, Object odat) {}


    class GrayFilter extends RGBImageFilter {
        public GrayFilter() {
            // The filter's operation does not depend on the
            // pixel's location, so IndexColorModels can be
            // filtered directly.
            canFilterIndexColorModel = true;
        }
        
        public int filterRGB(int x, int y, int rgb) {
            return ((rgb & 0x4f4f4fff));
        }
    }
}

