//
//  menuitem.java
//  Treehousestudio
//
//  Created by Maxwell Planck on Tue Jan 07 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

package treehouse;
import java.awt.*;
import java.applet.*;
import paint.*;

/**
 * HOW TO ADD A MENU ITEM
 *
 * This class is a generic class that should not be modified if you plan to add a menu item.  Instead, extend to this
 * class and overwrite it's makeMenuItem method.
 *
 * To make a new menu item add a static integer to your new THSMenuItems subclass with the name of your menuitem.  Assign
 * it a unique integer value (it doesn't matter what it is- do not to use 1000 since that is assigned to the spacer
 * menu item).  Add a case statement to the makeMenuItem() method with the unique static integer variable as the case
 * integer.  Have it return a newly instantiated menu item that you will add at the end of your class.  At the end of your
 * makeMenuItem method, for the default case, make sure to add the line:
 *
 *<code>return THSMenuItems.makeMenuItem(name, mainCanvas, g, app);</code>
 *
 * This will make sure any generic menu items are drawn if the case statement refers to them, like the spacer menu item.
 *
 * To make a menu item, have it extend the abstract class THSGeneralMenuItem. Make sure to instantiate the super class
 * with super("<name of menu item>") and to
 * specify an action for the actuate() method.  The <name of menu item> string specifies how the option appears
 * in the menu.  Note that you can name your menu item class any way you would like but for consistency, I
 * recommend naming it "<Name of Menu Item>MenuItem" (all capitals).
 *
 * Keep in mind that the makeMenuItem() method needs to be passed the objects on which the menu items act, so
 * if you need to add something that you would like to actuate upon with the menu item, pass it as a parameter during
 * instantiation.  For now, I believe all menu items should only effect the main canvas area (PaintCanvas class),
 * the Graphics object upon which everything is drawn, and the Applet itself.
 * Refer to the PaintMenuItems class already written as for my implementation of a paint canvas.
 *
 * Once you have done all of this, add the menu item to the menu of your choice by following the directions in the
 * THSApplet class.
 */

public class THSMenuItems {

    /***********************************************************************************
    ************************************************************************************
    ************************************************************************************
    **
    ** The spacer menu item is a general menu item that can be used in any application
    **
    ************************************************************************************
    ************************************************************************************
    ************************************************************************************/

    // These static final variables are used to reference the type of menu item to be instantiated
    // when the makeMenuItem() method is called.
    public final static int SPACER = 1000;

    /**
     * This method produces a menu item given a specific name (as a static integer variable).  It also takes in
     * the arguments that a menu item might use.  If the name specified is not a valid menu item, null is returned.
     */
    public static THSGeneralMenuItem makeMenuItem(int name, THSCanvas mainCanvas, Applet app) {
        switch (name) {
            case SPACER:
                return new Spacer();
            default:
                return null;
        }
    }

}

/**
* This unique type of menu item can be used to create separation within the menu item.  For instance, look
 * at the "Edit" menu in the paint applet and see the dividing line between the Undo action and the Cut action.
 * There is a Spacer menu item between the two.  Refer to the PaintApplet class to see how the code sets that up.
 */
class Spacer extends THSGeneralMenuItem {

    public Spacer() {
        super("-");
    }

    public void actuate() {
        // do nothing
        return;
    }
}
